from . import ModelSerializeMixin
from ..database import db


class Event(db.Model, ModelSerializeMixin):
    """Model for an Event"""
    __tablename__ = "event"
    id = db.Column(db.Integer, primary_key=True)
    begin = db.Column(db.DateTime, nullable=False)
    end = db.Column(db.DateTime)
    description = db.Column(db.String(240))
    kind = db.relationship("EventKind")
    slots = db.relationship("EventSlot", back_populates="event", cascade="all, delete")
    # notices = db.relationship("EventNotice", back_populates="event")

    _kind_id = db.Column(db.Integer, db.ForeignKey("event_kind.id", ondelete="CASCADE"), nullable=False)


class EventKind(db.Model, ModelSerializeMixin):
    """Model for an EventKind"""
    __tablename__ = "event_kind"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(30), nullable=False, unique=True)


class EventSlot(db.Model, ModelSerializeMixin):
    """Model for an EventSlot"""
    __tablename__ = "event_slot"
    id = db.Column(db.Integer, primary_key=True)
    start = db.Column(db.DateTime)
    end = db.Column(db.DateTime)
    slots = db.relationship("JobSlot", back_populates="event_slot")

    _event_id = db.Column(db.Integer, db.ForeignKey("event.id"), nullable=False)
    _event = db.relationship("Event", back_populates="slots")


class JobSlot(db.Model, ModelSerializeMixin):
    __tablename__ = "job_slot"
    id = db.Column(db.Integer, primary_key=True)
    needed_persons = db.Column(db.Numeric(precision=4, scale=2))
    kind = db.relationship("JobKind")
    jobs = db.relationship("Job", back_populates="slot")

    _event_slot_id = db.Column(db.Integer, db.ForeignKey("event_slot.id"))
    _event_slot = db.relationship("EventSlot", back_populates="slots")
    _kind_id = db.Column(db.Integer, db.ForeignKey("job_kind.id"))


class Job(db.Model, ModelSerializeMixin):
    __tablename__ = "job"
    user = db.relationship("User")
    value = db.Column(db.Numeric(precision=3, scale=2))

    _user_id = db.Column(db.Integer, db.ForeignKey("user.id"))
    _slot_id = db.Column(db.Integer, db.ForeignKey("job_slot.id"))
    _slot = db.relationship("JobSlot")
    _id = db.Column(db.Integer, primary_key=True)


class JobKind(db.Model, ModelSerializeMixin):
    __tablename__ = "job_kind"
    name = db.Column(db.String(30), nullable=False, unique=True)

    _id = db.Column(db.Integer, primary_key=True)
